/*
 *  Objects
 */

'use strict';

// Object literals --------------------------------------------------

{ // An empty unnamed object (useless). An object literal.
}

// Reference to object literal (a map of key/values)
var myObject = {
    val: 7, // property:value, note comma
    str: 'qwerty'
};

console.log(myObject);

// Two diffrent retrivals (everything public!!!)
console.log(myObject.str);
console.log(myObject['str']); // For computed keys

// Objects are like maps in Java (key/value).
// Loop through object properties
for (var prop in myObject) {
    console.log(prop + ":" + myObject[prop]);
}

// Objects are dynamic, add/remove property
myObject.other = 88;
console.log(myObject);
delete myObject.other; // Remove property
console.log(myObject.other);

// Subobject as values
var outer = {
    a: {
        aVal: 11
    },
    b: {
        bVal: 22
    },
    arr: [1, 2, 3, 4, 5]
};

console.log(outer.a.aVal);
console.log(outer.b.bVal);
console.log(outer.arr[2]);

// All objects have a prototype chain ----------------------------------------
// Displayed as __proto__ in Chrome (internal property)

/*
 *  NOTE: Her we use the **objects prototype** (internal pointer to ancestor object)
 *  NOT **prototype property**, totally different concept, see below
 */
console.log(Object.getPrototypeOf(outer) === Object.prototype);
console.log(Object.getPrototypeOf(outer.a) === Object.prototype);
console.log(Object.getPrototypeOf(Object.getPrototypeOf(outer.a)) === null);

console.log(typeof myObject); // Get a string idication type of param
console.log(typeof Object); // :-)
console.log(typeof Object.prototype);
console.log(typeof outer.a);
console.log(typeof outer.arr);

// Instantiation using Object.create ----------------------------------

var animal = {
    type: "unknown",
    displayType: function() { // Method which will display type of Animal
        console.log(this.type); // More on "this" later, it's not what you think
    }
};

// Create new animal type called lobster with _proto_ animal
var lobster = Object.create(animal);
lobster.type = "Invertebrate";
lobster.displayType(); // Output:Invertebrates

// Create new animal type called Fishes
var fish = Object.create(animal);
fish.type = "Fishes";
fish.displayType(); // Output:Fishes

console.log(Object.getPrototypeOf(lobster) === animal);
console.log(Object.getPrototypeOf(fish) === animal);

// Using a constructor function and new to instantiate ------------------------------
// (this may affect your mental health ...)

// A constructor function. Nothing special
// (leading upper case just an idiom, do use!)
function MyConstructor() {}

// Check the basic setup (see slides)
// Here we aslo use the *prototype property* of function-objects
console.log(Object.getPrototypeOf(MyConstructor) === Function.prototype);
// Remeber: Object is a function
console.log(Object.getPrototypeOf(MyConstructor.prototype) === Object.prototype);
console.log(Object.getPrototypeOf(Function.prototype) === Object.prototype);
console.log(MyConstructor.prototype.constructor === MyConstructor);

// Use constructor function to create object
var myObject = new MyConstructor();
console.log(Object.getPrototypeOf(myObject) === MyConstructor.prototype);

// Real sample
// Constructor function
function Person(name) {
    this.name = name || "unknown";
    this.say = function() {
        console.log("Hello from " + this.name); // "this" is the actual object (not always the case)
    };
}
// Person is a function, has prototype property
Person.prototype.common = function() { // Prototype shared by all instances
    console.log("Hello from common");
};

// Use to instantiate
var p1 = new Person("Otto"); // Will set __proto__ to value of Person.prototype
p1.say();
p1.common();

var p2 = new Person("Fia");
p2.say();
p2.common();

console.log(p1.hasOwnProperty("say")); // True
console.log(p1.hasOwnProperty("common")); // False, it's in __proto__
