package orm.db;

import orm.model.Person;
import java.io.Serializable;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Query;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

/**
 *  Base class for any DAO
 * @author hajo
 * @param <T> Data type
 * @param <K> Primary type
 */
public class AbstractDAO<T,K> implements Serializable, IDAO<T, K> {

    private EntityManagerFactory emf = null;
    private final Class<T> clazz;
    
    public AbstractDAO(EntityManagerFactory emf, Class<T> clazz) {
        this.emf = emf;
        this.clazz = clazz;
    }
   
    protected EntityManager getEntityManager() {
        return emf.createEntityManager();
    }

    // Writes must use tranactions (if not using EJBs)
    @Override
    public void add(T t) {
        /*
           The pattern for em is: get, use, close, throw away
        */
        EntityManager em = getEntityManager();  // Get 
        em.getTransaction().begin();
        em.persist(t);                         // Use
        em.getTransaction().commit();
        em.close();                       // Close, throw away
    }

    @Override
    public void update(T t) {
        EntityManager em = getEntityManager();
        em.getTransaction().begin();
        em.merge(t);
        em.getTransaction().commit();
        em.close();
    }

    @Override
    public void delete(K id) {
        EntityManager em = getEntityManager();
        em.getTransaction().begin();
        T t = em.getReference(clazz, id);
            em.remove(t);
        em.getTransaction().commit();
        em.close();
    }

    @Override
    public List<T> get() {
        return get(true, -1, -1);
    }

    @Override
    public List<T> get(int maxResults, int firstResult) {
        return get(false, maxResults, firstResult);
    }

    private List<T> get(boolean all, int maxResults, int firstResult) {
        EntityManager em = getEntityManager();
        CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
        cq.select(cq.from(Person.class));
        Query q = em.createQuery(cq);
        if (!all) {
            q.setMaxResults(maxResults);
            q.setFirstResult(firstResult);
        }
        List<T> result = q.getResultList();
        em.close();
        return result;
    }

    @Override
    public T getById(K id) {
        EntityManager em = getEntityManager();
        T result = em.find(clazz, id);
        em.close();
        return result;
    }

    @Override
    public int size() {
        EntityManager em = getEntityManager();
        CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
        Root<Person> rt = cq.from(Person.class);
        cq.select(em.getCriteriaBuilder().count(rt));
        Query q = em.createQuery(cq);
        int result = ((Long) q.getSingleResult()).intValue();
        em.close();
        return result;

    }

   
}
